@@ -59,6 +59,7 @@
 (require 'mail-utils)
 (require 'org-agenda)
 (require 'org-capture)
+(require 'ol)
 
 (defgroup org-contacts nil
   "Options about contacts management."
@@ -1013,7 +1014,7 @@ to do our best."
 			(setq phones-list (org-contacts-remove-ignored-property-values ignore-list (org-contacts-split-property tel)))
 			(setq result "")
 			(while phones-list
-			  (setq result (concat result  "TEL:" (org-link-unescape (org-contacts-strip-link (car phones-list))) "\n"))
+			  (setq result (concat result  "TEL:" (org-contacts-strip-link (org-link-unescape (car phones-list))) "\n"))
 			  (setq phones-list (cdr phones-list)))
 			result))
 	    (when bday
