@@ -40,6 +40,7 @@
 ;;   - OPENING: see `org-koma-letter-opening',
 ;;   - PHONE_NUMBER: see `org-koma-letter-phone-number',
 ;;   - URL: see `org-koma-letter-url',
+;;   - FROM_LOGO: see `org-koma-letter-from-logo',
 ;;   - SIGNATURE: see `org-koma-letter-signature',
 ;;   - PLACE: see `org-koma-letter-place',
 ;;   - LOCATION: see `org-koma-letter-location',
@@ -59,6 +60,7 @@
 ;;   - foldmarks (see `org-koma-letter-use-foldmarks')
 ;;   - phone (see `org-koma-letter-use-phone')
 ;;   - url (see `org-koma-letter-use-url')
+;;   - from-logo (see `org-koma-letter-use-from-logo')
 ;;   - email (see `org-koma-letter-use-email')
 ;;   - place (see `org-koma-letter-use-place')
 ;;   - location (see `org-koma-letter-use-location')
@@ -75,6 +77,43 @@
 ;;     (See also `org-koma-letter-author'),
 ;;   - EMAIL: Same as AUTHOR. (see also `org-koma-letter-email'),
 ;;
+;; FROM_LOGO uses LaTeX markup.  FROM_LOGO provides the
+;; "includegraphics" command to tell LaTeX where to find the logo.
+;; This command needs to know the logo's directory and file name.  The
+;; directory can either be relative or absolute, just as you would
+;; expect.  LaTeX can use three file types for the logo: PDF, JPEG, or
+;; PNG.  The logo can either include or exclude its extension, which
+;; might surprise you.  When you exclude its extension, LaTeX will
+;; search the directory for the "best" quality graphics format.  For
+;; example if it finds both logo.pdf and logo.png then it will
+;; identify the PDF as "better", and include "logo.pdf".  This can be
+;; useful, for example, when you are mocking up a logo in the PNG
+;; raster format and then switch over to the higher quality PDF vector
+;; format.  When you include the file extension then LaTeX will
+;; include it without searching for higher quality file types.
+;; Whatever file type you choose, it will probably require a few
+;; design iterations to get the best looking logo size for your
+;; letter.  Finally, the directory and file name are specified
+;; *without* quotes.  Here are some examples with commentary, in the
+;; location of your letter, with a logo named "logo", to get you
+;; started:
+;;
+;;   Logo in the same directory: \includegraphics{logo}
+;;       or a sub-directory:     \includegraphics{logos/production/logo}
+;;
+;;   Logos specified using absolute paths on Linux or Windows:
+;;
+;;       \includegraphics{~/correspondence/logo}
+;;       \includegraphics{~/correspondence/logos/production/logo}
+;;       \includegraphics{c:/you/correspondence/logo}
+;;       \includegraphics{c:/you/correspondence/logos/production/logo}
+;;
+;;   Logos in the same directory where the "better" quality PDF will
+;;   be chosen over the JPG:
+;;
+;;       \includegraphics{logo.pdf}
+;;       \includegraphics{logo.png}
+;;
 ;; Headlines are in general ignored.  However, headlines with special
 ;; tags can be used for specified contents like postscript (ps),
 ;; carbon copy (cc), enclosures (encl) and code to be inserted after
@@ -123,12 +162,13 @@
 
 ;;; Code:
 
+(require 'cl-lib)
 (require 'ox-latex)
 
 ;; Install a default letter class.
 (unless (assoc "default-koma-letter" org-latex-classes)
   (add-to-list 'org-latex-classes
-	       '("default-koma-letter" "\\documentclass[11pt]{scrlttr2}")))
+               '("default-koma-letter" "\\documentclass[11pt]{scrlttr2}")))
 
 
 ;;; User-Configurable Variables
@@ -155,9 +195,9 @@ Functions must return a string.
 This option can also be set with the AUTHOR keyword."
   :group 'org-export-koma-letter
   :type '(radio (function-item user-full-name)
-		(string)
-		(function)
-		(const :tag "Do not export author" nil)))
+                (string)
+                (function)
+                (const :tag "Do not export author" nil)))
 
 (defcustom org-koma-letter-email 'org-koma-letter-email
   "Sender's email address.
@@ -169,9 +209,9 @@ a function may be given.  Functions must return a string.
 This option can also be set with the EMAIL keyword."
   :group 'org-export-koma-letter
   :type '(radio (function-item org-koma-letter-email)
-		(string)
-		(function)
-		(const :tag "Do not export email" nil)))
+                (string)
+                (function)
+                (const :tag "Do not export email" nil)))
 
 (defcustom org-koma-letter-from-address ""
   "Sender's address, as a string.
@@ -193,6 +233,13 @@ This option can also be set with the URL keyword."
   :type 'string
   :safe #'stringp)
 
+(defcustom org-koma-letter-from-logo ""
+  "Commands for inserting the sender's logo, e. g., \\includegraphics{logo}.
+This option can also be set with the FROM_LOGO keyword."
+  :group 'org-export-koma-letter
+  :type 'string
+  :safe #'stringp)
+
 (defcustom org-koma-letter-place ""
   "Place from which the letter is sent, as a string.
 This option can also be set with the PLACE keyword."
@@ -296,14 +343,14 @@ This option can also be set with the OPTIONS keyword, e.g.:
     (const :tag "No export" nil)
     (const :tag "Default options" t)
     (set :tag "Configure options"
-	 (const :tag "Subject after opening" afteropening)
-	 (const :tag "Subject before opening" beforeopening)
-	 (const :tag "Subject centered" centered)
-	 (const :tag "Subject left-justified" left)
-	 (const :tag "Subject right-justified" right)
-	 (const :tag "Add title or description to subject" underlined)
-	 (const :tag "Set subject underlined" titled)
-	 (const :tag "Do not add title or description to subject" untitled)))
+         (const :tag "Subject after opening" afteropening)
+         (const :tag "Subject before opening" beforeopening)
+         (const :tag "Subject centered" centered)
+         (const :tag "Subject left-justified" left)
+         (const :tag "Subject right-justified" right)
+         (const :tag "Add title or description to subject" underlined)
+         (const :tag "Set subject underlined" titled)
+         (const :tag "Do not add title or description to subject" untitled)))
   :group 'org-export-koma-letter)
 
 (defcustom org-koma-letter-use-backaddress nil
@@ -345,24 +392,24 @@ This option can also be set with the OPTIONS keyword, e.g.:
 \"foldmarks:(b l m t)\"."
   :group 'org-export-koma-letter
   :type '(choice
-	  (const :tag "Activate default folding marks" t)
-	  (const :tag "Deactivate folding marks" nil)
-	  (set
-	   :tag "Configure folding marks"
-	   (const :tag "Activate upper horizontal mark on left paper edge" B)
-	   (const :tag "Deactivate upper horizontal mark on left paper edge" b)
-	   (const :tag "Activate all horizontal marks on left paper edge" H)
-	   (const :tag "Deactivate all horizontal marks on left paper edge" h)
-	   (const :tag "Activate left vertical mark on upper paper edge" L)
-	   (const :tag "Deactivate left vertical mark on upper paper edge" l)
-	   (const :tag "Activate middle horizontal mark on left paper edge" M)
-	   (const :tag "Deactivate middle horizontal mark on left paper edge" m)
-	   (const :tag "Activate punch or center mark on left paper edge" P)
-	   (const :tag "Deactivate punch or center mark on left paper edge" p)
-	   (const :tag "Activate lower horizontal mark on left paper edge" T)
-	   (const :tag "Deactivate lower horizontal mark on left paper edge" t)
-	   (const :tag "Activate all vertical marks on upper paper edge" V)
-	   (const :tag "Deactivate all vertical marks on upper paper edge" v))))
+          (const :tag "Activate default folding marks" t)
+          (const :tag "Deactivate folding marks" nil)
+          (set
+           :tag "Configure folding marks"
+           (const :tag "Activate upper horizontal mark on left paper edge" B)
+           (const :tag "Deactivate upper horizontal mark on left paper edge" b)
+           (const :tag "Activate all horizontal marks on left paper edge" H)
+           (const :tag "Deactivate all horizontal marks on left paper edge" h)
+           (const :tag "Activate left vertical mark on upper paper edge" L)
+           (const :tag "Deactivate left vertical mark on upper paper edge" l)
+           (const :tag "Activate middle horizontal mark on left paper edge" M)
+           (const :tag "Deactivate middle horizontal mark on left paper edge" m)
+           (const :tag "Activate punch or center mark on left paper edge" P)
+           (const :tag "Deactivate punch or center mark on left paper edge" p)
+           (const :tag "Activate lower horizontal mark on left paper edge" T)
+           (const :tag "Deactivate lower horizontal mark on left paper edge" t)
+           (const :tag "Activate all vertical marks on upper paper edge" V)
+           (const :tag "Deactivate all vertical marks on upper paper edge" v))))
 
 (defcustom org-koma-letter-use-phone nil
   "Non-nil prints sender's phone number.
@@ -379,6 +426,14 @@ This option can also be set with the OPTIONS keyword, e.g.:
   :type 'boolean
   :safe #'booleanp)
 
+(defcustom org-koma-letter-use-from-logo nil
+  "Non-nil prints sender's FROM_LOGO.
+This option can also be set with the OPTIONS keyword, e.g.:
+\"from-logo:t\"."
+  :group 'org-export-koma-letter
+  :type 'boolean
+  :safe #'booleanp)
+
 (defcustom org-koma-letter-use-email nil
   "Non-nil prints sender's email address.
 This option can also be set with the OPTIONS keyword, e.g.:
@@ -410,8 +465,8 @@ See also `org-koma-letter-opening' and
   "Non-nil means title should be interpreted as subject if subject is missing.
 This option can also be set with the OPTIONS keyword,
 e.g. \"title-subject:t\"."
-    :group 'org-export-koma-letter
-    :type 'boolean)
+  :group 'org-export-koma-letter
+  :type 'boolean)
 
 (defconst org-koma-letter-special-tags-in-letter '(to from closing location)
   "Header tags related to the letter itself.")
@@ -429,8 +484,8 @@ e.g. \"title-subject:t\"."
   "Holds special content temporarily.")
 
 (make-obsolete-variable 'org-koma-letter-use-title
-			'org-export-with-title
-			"25.1" 'set)
+                        'org-export-with-title
+                        "25.1" 'set)
 
 
 ;;; Define Back-End
@@ -444,6 +499,7 @@ e.g. \"title-subject:t\"."
     (:from-address "FROM_ADDRESS" nil org-koma-letter-from-address newline)
     (:phone-number "PHONE_NUMBER" nil org-koma-letter-phone-number)
     (:url "URL" nil org-koma-letter-url)
+    (:from-logo "FROM_LOGO" nil org-koma-letter-from-logo)
     (:email "EMAIL" nil (org-koma-letter--get-value org-koma-letter-email) t)
     (:to-address "TO_ADDRESS" nil nil newline)
     (:place "PLACE" nil org-koma-letter-place)
@@ -456,14 +512,15 @@ e.g. \"title-subject:t\"."
     (:special-tags-as-macro nil nil org-koma-letter-special-tags-as-macro)
     (:special-tags-in-letter nil nil org-koma-letter-special-tags-in-letter)
     (:special-tags-after-closing nil "after-closing-order"
-				 org-koma-letter-special-tags-after-closing)
+                                 org-koma-letter-special-tags-after-closing)
     (:special-tags-after-letter nil "after-letter-order"
-				org-koma-letter-special-tags-after-letter)
+                                org-koma-letter-special-tags-after-letter)
     (:with-backaddress nil "backaddress" org-koma-letter-use-backaddress)
     (:with-email nil "email" org-koma-letter-use-email)
     (:with-foldmarks nil "foldmarks" org-koma-letter-use-foldmarks)
     (:with-phone nil "phone" org-koma-letter-use-phone)
     (:with-url nil "url" org-koma-letter-use-url)
+    (:with-from-logo nil "from-logo" org-koma-letter-use-from-logo)
     (:with-place nil "place" org-koma-letter-use-place)
     (:with-subject nil "subject" org-koma-letter-subject-format)
     (:with-title-as-subject nil "title-subject" org-koma-letter-prefer-subject)
@@ -476,6 +533,7 @@ e.g. \"title-subject:t\"."
     (:inbuffer-email "EMAIL" nil 'koma-letter:empty)
     (:inbuffer-phone-number "PHONE_NUMBER" nil 'koma-letter:empty)
     (:inbuffer-url "URL" nil 'koma-letter:empty)
+    (:inbuffer-from-logo "FROM_LOGO" nil 'koma-letter:empty)
     (:inbuffer-place "PLACE" nil 'koma-letter:empty)
     (:inbuffer-location "LOCATION" nil 'koma-letter:empty)
     (:inbuffer-signature "SIGNATURE" nil 'koma-letter:empty)
@@ -484,21 +542,22 @@ e.g. \"title-subject:t\"."
     (:inbuffer-with-foldmarks nil "foldmarks" 'koma-letter:empty)
     (:inbuffer-with-phone nil "phone" 'koma-letter:empty)
     (:inbuffer-with-url nil "url" 'koma-letter:empty)
+    (:inbuffer-with-from-logo nil "from-logo" 'koma-letter:empty)
     (:inbuffer-with-place nil "place" 'koma-letter:empty))
   :translate-alist '((export-block . org-koma-letter-export-block)
-			(export-snippet . org-koma-letter-export-snippet)
-			(headline . org-koma-letter-headline)
-			(keyword . org-koma-letter-keyword)
-			(template . org-koma-letter-template))
+                     (export-snippet . org-koma-letter-export-snippet)
+                     (headline . org-koma-letter-headline)
+                     (keyword . org-koma-letter-keyword)
+                     (template . org-koma-letter-template))
   :menu-entry
   '(?k "Export with KOMA Scrlttr2"
        ((?L "As LaTeX buffer" org-koma-letter-export-as-latex)
-	(?l "As LaTeX file" org-koma-letter-export-to-latex)
-	(?p "As PDF file" org-koma-letter-export-to-pdf)
-	(?o "As PDF file and open"
-	    (lambda (a s v b)
-	      (if a (org-koma-letter-export-to-pdf t s v b)
-		(org-open-file (org-koma-letter-export-to-pdf nil s v b))))))))
+        (?l "As LaTeX file" org-koma-letter-export-to-latex)
+        (?p "As PDF file" org-koma-letter-export-to-pdf)
+        (?o "As PDF file and open"
+            (lambda (a s v b)
+              (if a (org-koma-letter-export-to-pdf t s v b)
+                (org-open-file (org-koma-letter-export-to-pdf nil s v b))))))))
 
 
 
@@ -515,7 +574,7 @@ e.g. \"title-subject:t\"."
   "Get contents from a headline tagged with KEY.
 The contents is stored in `org-koma-letter-special-contents'."
   (let ((value (cdr (assoc-string (org-koma-letter--get-value key)
-				  org-koma-letter-special-contents))))
+                                  org-koma-letter-special-contents))))
     (when value (org-string-nw-p (org-trim value)))))
 
 (defun org-koma-letter--get-value (value)
@@ -524,26 +583,26 @@ Determines if VALUE is nil, a string, a function or a symbol and
 return a string or nil."
   (when value
     (cond ((stringp value) value)
-	  ((functionp value) (funcall value))
-	  ((symbolp value) (symbol-name value))
-	  (t value))))
+          ((functionp value) (funcall value))
+          ((symbolp value) (symbol-name value))
+          (t value))))
 
 (defun org-koma-letter--special-contents-inline (keywords info)
   "Process KEYWORDS members of `org-koma-letter-special-contents'.
+
 KEYWORDS is a list of symbols.  Return them as a string to be
 formatted.
 
 The function is used for inserting content of special headings
-such as the one tagged with PS.
-"
+such as the one tagged with PS."
   (mapconcat
    (lambda (keyword)
      (let* ((name (org-koma-letter--get-value keyword))
-	    (value (org-koma-letter--get-tagged-contents name))
-	    (macrop (memq keyword (plist-get info :special-tags-as-macro))))
+            (value (org-koma-letter--get-tagged-contents name))
+            (macrop (memq keyword (plist-get info :special-tags-as-macro))))
        (cond ((not value) nil)
-	     (macrop (format "\\%s{%s}\n" name value))
-	     (t value))))
+             (macrop (format "\\%s{%s}\n" name value))
+             (t value))))
    keywords
    "\n"))
 
@@ -564,7 +623,8 @@ such as the one tagged with PS.
   "Transcode an EXPORT-BLOCK element into KOMA Scrlttr2 code.
 CONTENTS is nil.  INFO is a plist used as a communication
 channel."
-  (when (member (org-element-property :type export-block) '("KOMA-LETTER" "LATEX"))
+  (when (member (org-element-property :type export-block)
+		'("KOMA-LETTER" "LATEX"))
     (org-remove-indentation (org-element-property :value export-block))))
 
 ;;;; Export Snippet
@@ -583,7 +643,7 @@ channel."
 CONTENTS is nil.  INFO is a plist used as a communication
 channel."
   (let ((key (org-element-property :key keyword))
-	(value (org-element-property :value keyword)))
+        (value (org-element-property :value keyword)))
     ;; Handle specifically KOMA-LETTER keywords.  Otherwise, fallback
     ;; to `latex' back-end.
     (if (equal key "KOMA-LETTER") value
@@ -602,7 +662,7 @@ stored in `org-koma-letter-special-contents' and included at the
 appropriate place."
   (let ((special-tag (org-koma-letter--special-tag headline info)))
     (if (not special-tag)
-	contents
+        contents
       (push (cons special-tag contents) org-koma-letter-special-contents)
       "")))
 
@@ -611,13 +671,11 @@ appropriate place."
 INFO is a plist holding contextual information.  Return first
 special tag headline."
   (let ((special-tags (append
-		       (plist-get info :special-tags-in-letter)
-		       (plist-get info :special-tags-after-closing)
-		       (plist-get info :special-tags-after-letter))))
-    (catch 'exit
-      (dolist (tag (org-export-get-tags headline info))
-	(let ((tag (assoc-string tag special-tags)))
-	  (when tag (throw 'exit tag)))))))
+                       (plist-get info :special-tags-in-letter)
+                       (plist-get info :special-tags-after-closing)
+                       (plist-get info :special-tags-after-letter))))
+    (cl-some (lambda (tag) (and (assoc-string tag special-tags) tag))
+	     (org-export-get-tags headline info))))
 
 (defun org-koma-letter--keyword-or-headline (plist-key pred info)
   "Return the correct version of opening or closing.
@@ -628,15 +686,15 @@ and an info plist.  INFO is a plist holding contextual
 information.  Return the preferred candidate for the exported of
 PLIST-KEY."
   (let* ((keyword-candidate (plist-get info plist-key))
-	 (headline-candidate (when (and (plist-get info :with-headline-opening)
-					(or (plist-get info :special-headings)
-					    (not keyword-candidate)))
-			       (org-element-map (plist-get info :parse-tree)
-				   'headline
-				 (lambda (head)
-				   (when (funcall pred head info)
-				     (org-element-property :title head)))
-				 info t))))
+         (headline-candidate (when (and (plist-get info :with-headline-opening)
+                                        (or (plist-get info :special-headings)
+                                            (not keyword-candidate)))
+                               (org-element-map (plist-get info :parse-tree)
+                                   'headline
+                                 (lambda (h)
+                                   (and (funcall pred h info)
+					(org-element-property :title h)))
+                                 info t))))
     (org-export-data (or headline-candidate keyword-candidate "") info)))
 
 ;;;; Template
@@ -660,35 +718,35 @@ holding export options."
    ;; settings coming from buffer keywords.
    (org-koma-letter--build-settings 'global info)
    (mapconcat (lambda (file) (format "\\LoadLetterOption{%s}\n" file))
-	      (split-string (or (plist-get info :lco) ""))
-	      "")
+              (split-string (or (plist-get info :lco) ""))
+              "")
    (org-koma-letter--build-settings 'buffer info)
    ;; Date.
    (format "\\date{%s}\n" (org-export-data (org-export-get-date info) info))
    ;; Hyperref, document start, and subject and title.
    (let* ((with-subject (plist-get info :with-subject))
-	  (with-title (plist-get info :with-title))
-	  (title-as-subject (and with-subject
-				 (plist-get info :with-title-as-subject)))
-	  (subject* (org-string-nw-p
-		     (org-export-data (plist-get info :subject) info)))
-	  (title* (and with-title
-		       (org-string-nw-p
-			(org-export-data (plist-get info :title) info))))
-	  (subject (cond ((not with-subject) nil)
-			 (title-as-subject (or subject* title*))
-			 (t subject*)))
-	  (title (cond ((not with-title) nil)
-		       (title-as-subject (and subject* title*))
-		       (t title*)))
-	  (hyperref-template (plist-get info :latex-hyperref-template))
-	  (spec (append (list (cons ?t (or title subject "")))
-			(org-latex--format-spec info))))
+          (with-title (plist-get info :with-title))
+          (title-as-subject (and with-subject
+                                 (plist-get info :with-title-as-subject)))
+          (subject* (org-string-nw-p
+                     (org-export-data (plist-get info :subject) info)))
+          (title* (and with-title
+                       (org-string-nw-p
+                        (org-export-data (plist-get info :title) info))))
+          (subject (cond ((not with-subject) nil)
+                         (title-as-subject (or subject* title*))
+                         (t subject*)))
+          (title (cond ((not with-title) nil)
+                       (title-as-subject (and subject* title*))
+                       (t title*)))
+          (hyperref-template (plist-get info :latex-hyperref-template))
+          (spec (append (list (cons ?t (or title subject "")))
+                        (org-latex--format-spec info))))
      (concat
       (when (and with-subject (not (eq with-subject t)))
-	(format "\\KOMAoption{subject}{%s}\n"
-		(if (symbolp with-subject) with-subject
-		  (mapconcat #'symbol-name with-subject ","))))
+        (format "\\KOMAoption{subject}{%s}\n"
+                (if (symbolp with-subject) with-subject
+                  (mapconcat #'symbol-name with-subject ","))))
       ;; Hyperref.
       (format-spec hyperref-template spec)
       ;; Document start.
@@ -699,25 +757,30 @@ holding export options."
       (when (or (org-string-nw-p title) (org-string-nw-p subject)) "\n")))
    ;; Letter start.
    (let ((keyword-val (plist-get info :to-address))
-	 (heading-val (org-koma-letter--get-tagged-contents 'to)))
+         (heading-val (org-koma-letter--get-tagged-contents 'to)))
      (format "\\begin{letter}{%%\n%s}\n\n"
-	     (org-koma-letter--add-latex-newlines
-	      (or (if (plist-get info :special-headings)
-		      (or heading-val keyword-val)
-		    (or keyword-val heading-val))
-		  "\\mbox{}"))))
+             (org-koma-letter--add-latex-newlines
+              (or (if (plist-get info :special-headings)
+                      (or heading-val keyword-val)
+                    (or keyword-val heading-val))
+                  "\\mbox{}"))))
    ;; Opening.
    (format "\\opening{%s}\n\n"
-	   (org-koma-letter--keyword-or-headline
-	    :opening (lambda (h i) (not (org-koma-letter--special-tag h i)))
-	    info))
+           (org-koma-letter--keyword-or-headline
+            :opening
+	    (lambda (h i)
+	      (not (org-koma-letter--special-tag h i)))
+            info))
    ;; Letter body.
    contents
    ;; Closing.
    (format "\\closing{%s}\n"
 	   (org-koma-letter--keyword-or-headline
-	    :closing (lambda (h i) (eq (org-koma-letter--special-tag h i)
-				  'closing))
+	    :closing
+	    (lambda (h i)
+	      (let ((special-tag (org-koma-letter--special-tag h i)))
+		(and special-tag
+		     (string= "closing" special-tag))))
 	    info))
    (org-koma-letter--special-contents-inline
     (plist-get info :special-tags-after-closing) info)
@@ -733,23 +796,21 @@ holding export options."
 SCOPE is either `global' or `buffer'.  INFO is a plist used as
 a communication channel."
   (let* ((check-scope
-	  (function
-	   ;; Non-nil value when SETTING was defined in SCOPE.
-	   (lambda (setting)
-	     (let ((property (intern (format ":inbuffer-%s" setting))))
-	       (if (eq scope 'global)
-		   (eq (plist-get info property) 'koma-letter:empty)
-		 (not (eq (plist-get info property) 'koma-letter:empty)))))))
-	 (heading-or-key-value
-	  (function
-	   (lambda (heading key &optional scoped)
-	     (let* ((heading-val
-		     (org-koma-letter--get-tagged-contents heading))
-		    (key-val (org-string-nw-p (plist-get info key)))
-		    (scopedp (funcall check-scope (or scoped heading))))
-	       (and (or (and key-val scopedp) heading-val)
-		    (not (and (eq scope 'global) heading-val))
-		    (if scopedp key-val heading-val)))))))
+          ;; Non-nil value when SETTING was defined in SCOPE.
+          (lambda (setting)
+            (let ((property (intern (format ":inbuffer-%s" setting))))
+              (if (eq scope 'global)
+                  (eq (plist-get info property) 'koma-letter:empty)
+                (not (eq (plist-get info property) 'koma-letter:empty))))))
+         (heading-or-key-value
+          (lambda (heading key &optional scoped)
+            (let* ((heading-val
+                    (org-koma-letter--get-tagged-contents heading))
+                   (key-val (org-string-nw-p (plist-get info key)))
+                   (scopedp (funcall check-scope (or scoped heading))))
+              (and (or (and key-val scopedp) heading-val)
+                   (not (and (eq scope 'global) heading-val))
+                   (if scopedp key-val heading-val))))))
     (concat
      ;; Name.
      (let ((author (plist-get info :author)))
@@ -760,8 +821,8 @@ a communication channel."
      ;; From.
      (let ((from (funcall heading-or-key-value 'from :from-address)))
        (and from
-	    (format "\\setkomavar{fromaddress}{%s}\n"
-     		    (org-koma-letter--add-latex-newlines from))))
+            (format "\\setkomavar{fromaddress}{%s}\n"
+                    (org-koma-letter--add-latex-newlines from))))
      ;; Email.
      (let ((email (plist-get info :email)))
        (and email
@@ -786,45 +847,51 @@ a communication channel."
      (and (funcall check-scope 'with-url)
           (format "\\KOMAoption{fromurl}{%s}\n"
                   (if (plist-get info :with-url) "true" "false")))
+     ;; From Logo
+     (let ((from-logo (plist-get info :from-logo)))
+       (and (org-string-nw-p from-logo)
+            (funcall check-scope 'from-logo)
+            (format "\\setkomavar{fromlogo}{%s}\n" from-logo)))
+     (and (funcall check-scope 'with-from-logo)
+          (format "\\KOMAoption{fromlogo}{%s}\n"
+                  (if (plist-get info :with-from-logo) "true" "false")))
      ;; Signature.
      (let* ((heading-val
-	     (and (plist-get info :with-headline-opening)
-		  (org-string-nw-p
-		   (org-trim
-		    (org-export-data
-		     (org-koma-letter--get-tagged-contents 'closing)
-		     info)))))
-	    (signature (org-string-nw-p (plist-get info :signature)))
-	    (signature-scope (funcall check-scope 'signature)))
+             (and (plist-get info :with-headline-opening)
+                  (pcase (org-koma-letter--get-tagged-contents 'closing)
+                    ((and (pred org-string-nw-p) closing) (org-trim closing))
+                    (_ nil))))
+            (signature (org-string-nw-p (plist-get info :signature)))
+            (signature-scope (funcall check-scope 'signature)))
        (and (or (and signature signature-scope)
-		heading-val)
-	    (not (and (eq scope 'global) heading-val))
-	    (format "\\setkomavar{signature}{%s}\n"
-		    (if signature-scope signature heading-val))))
+                heading-val)
+            (not (and (eq scope 'global) heading-val))
+            (format "\\setkomavar{signature}{%s}\n"
+                    (if signature-scope signature heading-val))))
      ;; Back address.
      (and (funcall check-scope 'with-backaddress)
           (format "\\KOMAoption{backaddress}{%s}\n"
                   (if (plist-get info :with-backaddress) "true" "false")))
      ;; Place.
      (let ((with-place-set (funcall check-scope 'with-place))
-	   (place-set (funcall check-scope 'place)))
+           (place-set (funcall check-scope 'place)))
        (and (or (and with-place-set place-set)
-		(and (eq scope 'buffer) (or with-place-set place-set)))
-	    (format "\\setkomavar{place}{%s}\n"
-		    (if (plist-get info :with-place) (plist-get info :place)
-		      ""))))
+                (and (eq scope 'buffer) (or with-place-set place-set)))
+            (format "\\setkomavar{place}{%s}\n"
+                    (if (plist-get info :with-place) (plist-get info :place)
+                      ""))))
      ;; Location.
      (let ((location (funcall heading-or-key-value 'location :location)))
        (and location
-	    (format "\\setkomavar{location}{%s}\n" location)))
+            (format "\\setkomavar{location}{%s}\n" location)))
      ;; Folding marks.
      (and (funcall check-scope 'with-foldmarks)
           (let ((foldmarks (plist-get info :with-foldmarks)))
-	    (cond ((consp foldmarks)
-		   (format "\\KOMAoptions{foldmarks=true,foldmarks=%s}\n"
-			   (mapconcat #'symbol-name foldmarks "")))
-		  (foldmarks "\\KOMAoptions{foldmarks=true}\n")
-		  (t "\\KOMAoptions{foldmarks=false}\n")))))))
+            (cond ((consp foldmarks)
+                   (format "\\KOMAoptions{foldmarks=true,foldmarks=%s}\n"
+                           (mapconcat #'symbol-name foldmarks "")))
+                  (foldmarks "\\KOMAoptions{foldmarks=true}\n")
+                  (t "\\KOMAoptions{foldmarks=false}\n")))))))
 
 
 
@@ -832,7 +899,7 @@ a communication channel."
 
 ;;;###autoload
 (defun org-koma-letter-export-as-latex
-  (&optional async subtreep visible-only body-only ext-plist)
+    (&optional async subtreep visible-only body-only ext-plist)
   "Export current buffer as a KOMA Scrlttr2 letter.
 
 If narrowing is active in the current buffer, only export its
@@ -854,7 +921,7 @@ contents of hidden elements.
 When optional argument BODY-ONLY is non-nil, only write code
 between \"\\begin{letter}\" and \"\\end{letter}\".
 
-EXT-PLIST, when provided, is a proeprty list with external
+EXT-PLIST, when provided, is a property list with external
 parameters overriding Org default settings, but still inferior to
 file-local settings.
 
@@ -869,7 +936,7 @@ non-nil."
 
 ;;;###autoload
 (defun org-koma-letter-export-to-latex
-  (&optional async subtreep visible-only body-only ext-plist)
+    (&optional async subtreep visible-only body-only ext-plist)
   "Export current buffer as a KOMA Scrlttr2 letter (tex).
 
 If narrowing is active in the current buffer, only export its
@@ -901,13 +968,13 @@ directory.
 Return output file's name."
   (interactive)
   (let ((outfile (org-export-output-file-name ".tex" subtreep))
-	(org-koma-letter-special-contents))
+        (org-koma-letter-special-contents))
     (org-export-to-file 'koma-letter outfile
       async subtreep visible-only body-only ext-plist)))
 
 ;;;###autoload
 (defun org-koma-letter-export-to-pdf
-  (&optional async subtreep visible-only body-only ext-plist)
+    (&optional async subtreep visible-only body-only ext-plist)
   "Export current buffer as a KOMA Scrlttr2 letter (pdf).
 
 If narrowing is active in the current buffer, only export its
@@ -936,7 +1003,7 @@ file-local settings.
 Return PDF file's name."
   (interactive)
   (let ((file (org-export-output-file-name ".tex" subtreep))
-	(org-koma-letter-special-contents))
+        (org-koma-letter-special-contents))
     (org-export-to-file 'koma-letter file
       async subtreep visible-only body-only ext-plist
       (lambda (file) (org-latex-compile file)))))
