@@ -1,10 +1,10 @@
 ;;; ob-ebnf.el --- Babel Functions for EBNF          -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2013-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2019 Free Software Foundation, Inc.
 
 ;; Author: Michael Gauland
 ;; Keywords: literate programming, reproducible research
-;; Homepage: http://orgmode.org
+;; Homepage: https://orgmode.org
 ;; Version: 1.00
 
 ;; This file is part of GNU Emacs.
@@ -49,8 +49,8 @@
 ;; Use ebnf-eps-buffer to produce an encapsulated postscript file.
 ;;
 (defun org-babel-execute:ebnf (body params)
-  "Execute a block of Ebnf code with org-babel.  This function is
-called by `org-babel-execute-src-block'"
+  "Execute a block of Ebnf code with org-babel.
+This function is called by `org-babel-execute-src-block'."
   (save-excursion
     (let* ((dest-file (cdr (assq :file params)))
 	   (dest-dir (file-name-directory dest-file))
