@@ -102,7 +102,7 @@
   (puthash session-name buffer org-babel-scheme-repl-map))
 
 (defun org-babel-scheme-get-buffer-impl (buffer)
-  "Returns the scheme implementation geiser associates with the buffer."
+  "Return the scheme implementation geiser associates with the buffer."
   (with-current-buffer (set-buffer buffer)
     geiser-impl--implementation))
 
@@ -131,7 +131,7 @@ org-babel-scheme-execute-with-geiser will use a temporary session."
 	(name)))
 
 (defmacro org-babel-scheme-capture-current-message (&rest body)
-  "Capture current message in both interactive and noninteractive mode"
+  "Capture current message in both interactive and noninteractive mode."
   `(if noninteractive
        (let ((original-message (symbol-function 'message))
              (current-message nil))
@@ -147,10 +147,11 @@ org-babel-scheme-execute-with-geiser will use a temporary session."
        (current-message))))
 
 (defun org-babel-scheme-execute-with-geiser (code output impl repl)
-  "Execute code in specified REPL. If the REPL doesn't exist, create it
-using the given scheme implementation.
+  "Execute code in specified REPL.
+If the REPL doesn't exist, create it using the given scheme
+implementation.
 
-Returns the output of executing the code if the output parameter
+Returns the output of executing the code if the OUTPUT parameter
 is true; otherwise returns the last value."
   (let ((result nil))
     (with-temp-buffer
@@ -198,7 +199,7 @@ Emacs-lisp table, otherwise return the results as a string."
 
 (defun org-babel-execute:scheme (body params)
   "Execute a block of Scheme code with org-babel.
-This function is called by `org-babel-execute-src-block'"
+This function is called by `org-babel-execute-src-block'."
   (let* ((source-buffer (current-buffer))
 	 (source-buffer-name (replace-regexp-in-string ;; zap surrounding *
 			      "^ ?\\*\\([^*]+\\)\\*" "\\1"
