@@ -1,6 +1,6 @@
 ;;; test-org-colview.el --- Tests for org-colview.el -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016  Nicolas Goaziou
+;; Copyright (C) 2016, 2019  Nicolas Goaziou
 
 ;; Author: Nicolas Goaziou <mail@nicolasgoaziou.fr>
 
@@ -142,8 +142,8 @@
   ;; Special case: width takes into account link narrowing in ITEM.
   (should
    (equal
-    '("* [123]" . 7)
-    (org-test-with-temp-text "* [[http://orgmode.org][123]]"
+    '("* 123" . 5)
+    (org-test-with-temp-text "* [[https://orgmode.org][123]]"
       (let ((org-columns-default-format "%ITEM")) (org-columns))
       (cons (get-char-property (point) 'org-columns-value-modified)
 	    (aref org-columns-current-maxwidths 0)))))
@@ -224,7 +224,7 @@
 :END:"
       (let ((org-columns-default-format "%A{+;%.1f}")) (org-columns))
       (get-char-property (point) 'org-columns-value-modified))))
-  ;; {:} sums times.  Plain numbers are hours.
+  ;; {:} sums times.  Plain numbers are minutes.
   (should
    (equal
     "4:10"
@@ -242,7 +242,7 @@
       (get-char-property (point) 'org-columns-value-modified))))
   (should
    (equal
-    "3:30"
+    "1:32"
     (org-test-with-temp-text
 	"* H
 ** S1
@@ -510,10 +510,7 @@
   (should
    (equal
     "0min"
-    (cl-letf (((symbol-function 'current-time)
-	       (lambda ()
-		 (apply #'encode-time
-			(org-parse-time-string "<2014-03-04 Tue>")))))
+    (org-test-at-time "<2014-03-04 Tue>"
       (org-test-with-temp-text
 	  "* H
 ** S1
@@ -529,10 +526,7 @@
   (should
    (equal
     "2d"
-    (cl-letf (((symbol-function 'current-time)
-	       (lambda ()
-		 (apply #'encode-time
-			(org-parse-time-string "<2014-03-04 Tue>")))))
+    (org-test-at-time "<2014-03-04 Tue>"
       (org-test-with-temp-text
 	  "* H
 ** S1
@@ -548,10 +542,7 @@
   (should
    (equal
     "1d 12h"
-    (cl-letf (((symbol-function 'current-time)
-	       (lambda ()
-		 (apply #'encode-time
-			(org-parse-time-string "<2014-03-04 Tue>")))))
+    (org-test-at-time "<2014-03-04 Tue>"
       (org-test-with-temp-text
 	  "* H
 ** S1
@@ -683,6 +674,56 @@
 	     '(("custom" . (lambda (s _) (mapconcat #'identity s "|")))))
 	    (org-columns-default-format "%A{custom}")) (org-columns))
       (get-char-property (point) 'org-columns-value-modified))))
+  ;; Allow custom _collect_ for summary types.
+  (should
+   (equal
+    "2"
+    (org-test-with-temp-text
+	"* H
+** S1
+:PROPERTIES:
+:A: 1
+:END:
+** S1
+:PROPERTIES:
+:A: 2
+:A-OK: 1
+:END:"
+     (let ((org-columns-summary-types
+	    '(("custom" org-columns--summary-sum
+	       (lambda (p)
+                 (if (equal "1" (org-entry-get nil (format "%s-OK" p)))
+		     (org-entry-get nil p)
+		   "")))))
+	   (org-columns-default-format "%A{custom}")) (org-columns))
+     (get-char-property (point) 'org-columns-value-modified))))
+  ;; Allow custom collect function to be used for different columns
+  (should
+   (equal
+    '("2" "1")
+    (org-test-with-temp-text
+     "* H
+** S1
+:PROPERTIES:
+:A: 1
+:B: 1
+:B-OK: 1
+:END:
+** S1
+:PROPERTIES:
+:A: 2
+:B: 2
+:A-OK: 1
+:END:"
+     (let ((org-columns-summary-types
+	    '(("custom" org-columns--summary-sum
+	       (lambda (p)
+                 (if (equal "1" (org-entry-get nil (format "%s-OK" p)))
+		     (org-entry-get nil p)
+		   "")))))
+	   (org-columns-default-format "%A{custom} %B{custom}")) (org-columns))
+     (list (get-char-property (point) 'org-columns-value-modified)
+	   (get-char-property (1+ (point)) 'org-columns-value-modified)))))
   ;; Allow multiple summary types applied to the same property.
   (should
    (equal
@@ -1420,6 +1461,26 @@
 ** H1.1
 :PROPERTIES:
 :A: 1
+:END:"
+      (let ((org-columns-default-format "%ITEM %A")) (org-update-dblock))
+      (buffer-substring-no-properties (point) (outline-next-heading)))))
+  ;; Test `:exclude-tags' parameter.
+  (should
+   (equal
+    "#+BEGIN: columnview :exclude-tags (\"excludeme\")
+| ITEM | A |
+|------+---|
+| H1   |   |
+#+END:
+"
+    (org-test-with-temp-text
+        "
+* H1
+<point>#+BEGIN: columnview :exclude-tags (\"excludeme\")
+#+END:
+** H1.1 :excludeme:
+:PROPERTIES:
+:A: 1
 :END:"
       (let ((org-columns-default-format "%ITEM %A")) (org-update-dblock))
       (buffer-substring-no-properties (point) (outline-next-heading)))))
@@ -1435,19 +1496,6 @@
         "* H\n<point>#+BEGIN: columnview :format \"%ITEM(Name)\"\n#+END:"
       (let ((org-columns-default-format "%ITEM")) (org-update-dblock))
       (buffer-substring-no-properties (point) (point-max)))))
-  ;; Test `:width' parameter
-  (should
-   (equal
-    "#+BEGIN: columnview :width t
-| ITEM       | A |
-|------------+---|
-| H          |   |
-| <10>       |   |
-#+END:"
-    (org-test-with-temp-text
-        "* H\n<point>#+BEGIN: columnview :width t\n#+END:"
-      (let ((org-columns-default-format "%10ITEM %A")) (org-update-dblock))
-      (buffer-substring-no-properties (point) (point-max)))))
   ;; When inserting ITEM values, make sure to clean sensitive
   ;; contents, like unique targets or forbidden inline src-blocks.
   (should
